Package: ecmwfr
Title: Interface to 'ECMWF' and 'CDS' Data Web Services
Version: 2.0.3
Authors@R: c(person(
               family = "Hufkens",
               given = "Koen",
               email = "koen.hufkens@gmail.com",
               role = c("aut", "cre", "cph"),
               comment = c(ORCID = "0000-0002-5070-8109")),
             person(
               family = "Stauffer",
               given = "Reto",
               email = "Reto.Stauffer@uibk.ac.at",
               role = c("ctb"),
               comment = c(ORCID = "0000-0002-3798-5507")),
             person(
               family = "Campitelli",
               given = "Elio",
               email = "elio.campitelli@cima.fcen.uba.ar",
               role = c("ctb"),
               comment = c(ORCID = "0000-0002-7742-9230")),
            person(
               given = "BlueGreen Labs",
               role = c("fnd"))
           )
Description: Programmatic interface to the European Centre for Medium-Range
    Weather Forecasts dataset web services (ECMWF; <https://www.ecmwf.int/>)
    and Copernicus's Data Stores. Allows for easy downloads of weather 
    forecasts and climate reanalysis data in R. Data stores covered include the Climate Data Store (CDS; 
    <https://cds.climate.copernicus.eu>), Atmosphere Data Store (ADS; 
    <https://ads.atmosphere.copernicus.eu>) and Early Warning Data Store (CEMS; 
    <https://ewds.climate.copernicus.eu>).
URL: https://github.com/bluegreen-labs/ecmwfr
BugReports: https://github.com/bluegreen-labs/ecmwfr/issues
Depends: R (>= 4.2)
Imports: httr, memoise, getPass, R6, keyring, tools
License: AGPL-3
ByteCompile: true
RoxygenNote: 7.3.1
Suggests: rmarkdown, covr, xml2, testthat, terra, maps, ncdf4, knitr,
        rlang, rstudioapi, jsonlite
VignetteBuilder: knitr
Encoding: UTF-8
NeedsCompilation: no
Packaged: 2025-02-10 11:28:47 UTC; khufkens
Author: Koen Hufkens [aut, cre, cph] (<https://orcid.org/0000-0002-5070-8109>),
  Reto Stauffer [ctb] (<https://orcid.org/0000-0002-3798-5507>),
  Elio Campitelli [ctb] (<https://orcid.org/0000-0002-7742-9230>),
  BlueGreen Labs [fnd]
Maintainer: Koen Hufkens <koen.hufkens@gmail.com>
Repository: CRAN
Date/Publication: 2025-02-10 12:00:02 UTC
Built: R 4.6.0; ; 2026-04-28 00:38:31 UTC; unix
