# Generated by roxygen2: do not edit by hand

S3method(print,ecmwfr_archetype)
export(wf_archetype)
export(wf_check_request)
export(wf_dataset_info)
export(wf_datasets)
export(wf_delete)
export(wf_get_key)
export(wf_request)
export(wf_request_batch)
export(wf_set_key)
export(wf_transfer)
import(tools)
importFrom(R6,R6Class)
importFrom(memoise,memoise)
importFrom(utils,browseURL)
